@@ -1,4 +1,4 @@
-#!/usr/bin/python
+#!/usr/bin/env python
 #
 # Util to download a full Flickr set.
 #
@@ -17,6 +17,8 @@ from flickr_api.flickrerrors import FlickrAPIError
 from dateutil import parser
 import yaml
 
+from flickr_download.filename_handlers import get_filename_handler
+
 CONFIG_FILE = "~/.flickr_download"
 OAUTH_TOKEN_FILE = "~/.flickr_token"
 
@@ -77,11 +79,12 @@ def _load_defaults():
     return {}
 
 
-def download_set(set_id, size_label=None):
+def download_set(set_id, get_filename, size_label=None):
     """
     Download the set with 'set_id' to the current directory.
 
     @param set_id: str, id of the photo set
+    @param get_filename: Function, function that creates a filename for the photo
     @param size_label: str|None, size to download (or None for largest available)
     """
     suffix = " ({})".format(size_label) if size_label else ""
@@ -99,9 +102,10 @@ def download_set(set_id, size_label=None):
             raise
 
     if not os.path.exists(pset.title):
-       os.mkdir(pset.title)
+        os.mkdir(pset.title)
+
     for photo in photos:
-        fname = '{0}/{1}{2}.jpg'.format(pset.title, photo.title, suffix)
+        fname = '{0}/{1}'.format(pset.title, get_filename(pset, photo, suffix))
         if os.path.exists(fname):
             # TODO: Ideally we should check for file size / md5 here
             # to handle failed downloads.
@@ -118,6 +122,20 @@ def download_set(set_id, size_label=None):
         os.utime(fname, (taken_unix, taken_unix))
 
 
+def download_user(username, get_filename, size_label):
+    """
+    Download all the sets owned by the given user.
+
+    @param username: str, username
+    @param get_filename: Function, function that creates a filename for the photo
+    @param size_label: str|None, size to download (or None for largest available)
+    """
+    user = Flickr.Person.findByUserName(username)
+    photosets = user.getPhotosets()
+    for photoset in photosets:
+        download_set(photoset.id, get_filename, size_label)
+
+
 def print_sets(username):
     """
     Print all sets for the given user
@@ -142,8 +160,12 @@ def main():
                         help='List photosets for a user')
     parser.add_argument('-d', '--download', type=str, metavar='SET_ID',
                         help='Download the given set')
+    parser.add_argument('-u', '--download_user', type=str, metavar='USERNAME',
+                        help='Download all sets for a given user')
     parser.add_argument('-q', '--quality', type=str, metavar='SIZE_LABEL',
                         default=None, help='Quality of the picture')
+    parser.add_argument('-n', '--naming', type=str, metavar='NAMING_MODE',
+                        default='title', help='Photo naming mode')
     parser.set_defaults(**_load_defaults())
 
     args = parser.parse_args()
@@ -158,8 +180,15 @@ def main():
 
     if args.list:
         print_sets(args.list)
-    elif args.download:
-        download_set(args.download, args.quality)
+    elif args.download or args.download_user:
+        try:
+            get_filename = get_filename_handler(args.naming)
+            if args.download:
+                download_set(args.download, get_filename, args.quality)
+            else:
+                download_user(args.download_user, get_filename, args.quality)
+        except KeyboardInterrupt:
+            print >> sys.stderr, 'Forcefully aborting. Last photo download might be partial :('
     else:
         print >> sys.stderr, 'ERROR: Must pass either --list or --download\n'
         parser.print_help()
